import processing.video.*;
////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Capture cam;
void SetupCamera()
{
  String[] cameras = Capture.list();
  if (cameras.length == 0) exit();
  cam = new Capture(this, cameras[0]);
  cam.start();
}

void InitCatPos()
{ 
  cat_pos[0] = new Pos(width*-0.15, height*0.8);
  cat_pos[1] = new Pos(width*0.05, height*0.8);
  cat_pos[2] = new Pos(width*0.25, height*0.8);
  cat_pos[3] = new Pos(width*0.5, height*0.5);
  cat_pos[4] = new Pos(width*0.75, height*0.35);
  cat_pos[5] = new Pos(width*0.95, height*0.35);
  cat_pos[6] = new Pos(width*1.15, height*0.35);
  flying_cat_pos1 = new Pos(width*-0.15, height*0.2);
  flying_cat_pos2 = new Pos(width*1.15, height*0.2);
}

void LoadImg()
{
  for(int i=0; i<cat_img_num;i++)
    cat_img[i] = loadImage(i+".png");
}

void InitFont()
{
  PFont font = loadFont("ComicSansMS-48.vlw");
  textFont(font, 40);
}

void setup()
{
  size(1440,1080);
  colorMode(HSB,255);
  SetupCamera();
  InitCatPos();
  LoadImg();
  InitFont();
  InitBGM();
}

void InitBGM(){
  bgm.file1 = new SoundFile(this, "sounds/bgm1.mp3");
}
//////////////////////////////////////////////////////////////////
int pgw = 240;
int pgh = 180;
PGraphics pg;
void DrawCam()
{
  cam.read();
  pg = createGraphics(pgw,pgh);
  pg.beginDraw();
  pg.image(cam,0,0,pgw,pgh);
  pg.endDraw();
}

float cbl = 0.1;
float babel_hue;
boolean babel_hue_needed = true;
boolean IsColorUnique()
{
  float sum_hue=0, sum_hue_sq=0;
  int pix_num = 0;

  pg.loadPixels();
  for (int x = (int)(pgw*(1-cbl)/2); x < (int)(pgw*(1+cbl)/2); x++)
    for (int y = (int)(pgh*(1-cbl)/2); y < (int)(pgh*(1+cbl)/2); y++){
      float pixel_hue = hue(pg.pixels[x + y*pgw]);
      sum_hue += pixel_hue;
      sum_hue_sq += pixel_hue*pixel_hue;
      pix_num++;
    }

  float V_hue = sum_hue_sq/pix_num - (sum_hue/pix_num)*(sum_hue/pix_num);
  println(V_hue);
  return V_hue < 70;
}
void SetupBabelColor()
{ 
  float sum_hue=0;
  int pix_num = 0;

  pg.loadPixels();
  for (int x = (int)(pgw*(1-cbl)/2); x < (int)(pgw*(1+cbl)/2); x++)
    for (int y = (int)(pgh*(1-cbl)/2); y < (int)(pgh*(1+cbl)/2); y++){
      float pixel_hue = hue(pg.pixels[x + y*pgw]);
      sum_hue += pixel_hue;
      pix_num++;
    }
  
  babel_hue = sum_hue/pix_num;
  println("hue = " + babel_hue);
  babel_hue_needed = false;
}

//////////////////////////////////////////////////////////////////
float glw = 0.01*pgw;
float right_babel_y=0, left_babel_y=0;
boolean isPixelBabel(int x, int y){  
  float diff = 0;
  int rad=1;
  int count = 0;
  for(int dx=-rad; dx<=rad; dx++)
    for(int dy=-rad; dy<=rad; dy++){
      if(x+dx<0 || x+dx>=pgw || y+dy<0 || y+dy>=pgh) continue;
      if(saturation(pg.pixels[(x+dx) + (y+dy)*pgw]) < 50) continue;
      if(brightness(pg.pixels[(x+dx) + (y+dy)*pgw]) < 50) continue;
      color pc = pg.pixels[(x+dx) + (y+dy)*pgw];
      diff += abs(hue(pc)-babel_hue);
      count++;
    }
  diff /= count;
  return diff<10;
}
void DrawGuideline()
{
  pg.beginDraw();
  pg.rectMode(CENTER);
  pg.fill(color(255),100);
  pg.noStroke();
  pg.rect(pgw/3, pgh/2, glw, pgh);
  pg.rect(pgw*2/3, pgh/2, glw, pgh);
  
  float left_babel_y_sum = 0;
  int left_babel_num = 0;
  for(int x=pgw/3; x<pgw/3 + 6; x++)
    for(int y=0; y<=pgh; y++)
      if(isPixelBabel(x, y)){
        left_babel_y_sum += y;
        left_babel_num++;
      }
  if(left_babel_num >= 6){
    left_babel_y = left_babel_y_sum / left_babel_num;
    pg.fill(255,0,0);
    pg.rect(pgw/3, left_babel_y, glw, pgh/10);
  }
  
  float right_babel_y_sum = 0;
  int right_babel_num = 0;
  for(int x=pgw*2/3; x<pgw*2/3 + 6; x++)
    for(int y=0; y<=pgh; y++)
      if(isPixelBabel(x, y)){
        right_babel_y_sum += y;
        right_babel_num++;
      }
  if(right_babel_num >= 6){
    right_babel_y = right_babel_y_sum / right_babel_num;
    pg.fill(255,0,0);
    pg.rect(pgw*2/3, right_babel_y, glw, pgh/10);
  }
  pg.endDraw();
}

//////////////////////////////////////////////////////////////////
void Display()
{
  pushMatrix();
  translate(width,0);
  scale(-1,1);
  image(pg,0,0,width,height);
  popMatrix();
}

void UpdateCenterPos()
{
  cat_pos[center_pos_index].y = (left_babel_y + right_babel_y)/2 * height/pgh;
}

void MakeCat()
{
  if(cat_pos[0].cat == null)
    cat_pos[0].cat = new Cat(0);
}

int migrated_cats = 0;
void RemoveCat()
{
  if(flying_cat_pos2.cat != null)
    flying_cat_pos2.cat = null;
  if(cat_pos[pos_num-1].cat != null){
    migrated_cats++;
    cat_pos[pos_num-1].cat = null;
    if(migrated_cats == 20)
      flying_cat_pos1.cat = new Cat(1);
    else if(migrated_cats == 40)
      flying_cat_pos1.cat = new Cat(2);
  }
}

void DrawMigratedCats()
{
  fill(255);
  textAlign(RIGHT);
  text(migrated_cats, width*0.85, height*0.95);
  imageMode(CENTER);
  image(cat_img[MIGRATED], width*0.9, height*0.925, width*0.1, height*0.1);
  imageMode(CORNER);
}

void draw()
{
  if(!cam.available()) return;
  DrawCam();
  
  
  if(babel_hue_needed) {
    Display();
    if(IsColorUnique()) {
      SetupBabelColor();
    }else {
      fill(color(255,0,0),120);
      rectMode(CENTER);
      rect(width*0.5,height*0.5,width*cbl, height*cbl);
    }
  }else {
    
    bgm.Loop1();
    DrawGuideline();
    Display();
    
    BG bg = new BG();
    bg.Draw();
    
    UpdateCenterPos();
    
    MakeCat();
    if(flying_cat_pos1.cat != null)
      flying_cat_pos1.cat.Draw();
    for(int i=pos_num-1; i>=0; i--)
      if(cat_pos[i].cat != null)
        cat_pos[i].cat.Draw();
    RemoveCat();
    if(migrated_cats > 0)
      DrawMigratedCats();
  }
}